'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deepRemove = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _immutable = require('immutable');

var _utils = require('../utils');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var deepRemove = exports.deepRemove = function deepRemove(map, id) {
  var children = map.getIn(['items', id, 'children']);
  if (children) children.forEach(function (c) {
    map = deepRemove(map, c);
  });
  return map.deleteIn(['items', id]);
};

var defaultItems = (0, _immutable.fromJS)({ root: {
    id: 'root',
    type: 'Column',
    props: {},
    children: [],
    style: {}
  } });

var LayoutState = function (_Record) {
  _inherits(LayoutState, _Record);

  function LayoutState() {
    _classCallCheck(this, LayoutState);

    var _this = _possibleConstructorReturn(this, (LayoutState.__proto__ || Object.getPrototypeOf(LayoutState)).call(this));

    _this.dragging = null;
    return _this;
  }

  _createClass(LayoutState, [{
    key: 'onChange',
    value: function onChange(listener) {
      this.listener = listener;
    }
  }, {
    key: 'getItem',
    value: function getItem(id) {
      return this.items.get(id);
    }
  }, {
    key: 'getItemJS',
    value: function getItemJS(id) {
      return this.items.get(id).toJS();
    }
  }, {
    key: 'addItem',
    value: function addItem(parentId, idx, item) {
      var nextState = void 0;
      if (item.id) {
        if (parentId === item.parent.id && idx > item.parent.idx) {
          idx--;
        }
        nextState = this.updateIn(['items', item.parent.id, 'children'], function (c) {
          return c.filter(function (id) {
            return id !== item.id;
          });
        }).updateIn(['items', parentId, 'children'], function (c) {
          return c.splice(idx, 0, item.id);
        }).setIn(['items', item.id, 'parent'], (0, _immutable.fromJS)({ id: parentId, idx: idx }));
      } else {
        item.id = (0, _utils.generateRandomKey)();
        item.parent = { id: parentId, idx: idx };
        nextState = this.setIn(['items', item.id], (0, _immutable.fromJS)(item)).updateIn(['items', parentId, 'children'], function (c) {
          return c.splice(idx, 0, item.id);
        });
      }
      this.listener(nextState);
    }
  }, {
    key: 'getAncestors',
    value: function getAncestors(id) {
      var result = [this.getItemJS(id)];
      while (result[0].parent && result.length < 4) {
        result.unshift(this.getItemJS(result[0].parent.id));
      }
      return result;
    }
  }, {
    key: 'removeItem',
    value: function removeItem(id) {
      if (id === 'root') return;
      var parent = this.getIn(['items', id, 'parent', 'id']);
      var nextState = this.updateIn(['items', parent, 'children'], function (c) {
        return c.filter(function (cId) {
          return cId !== id;
        });
      });
      this.listener(deepRemove(nextState, id));
    }
  }, {
    key: 'moveItem',
    value: function moveItem(from) {
      var _this2 = this;

      return function (to) {
        if (to.id === from.id && from.idx <= to.idx) to.idx -= 1;
        var item = void 0;
        var nextState = _this2.updateIn(['items', from.id, 'children'], function (c) {
          item = c.get(from.idx);
          return c.splice(from.idx, 1);
        }).updateIn(['items', to.id, 'children'], function (c) {
          return c.splice(to.idx, 0, item);
        });
        _this2.listener(nextState);
      };
    }
  }, {
    key: 'updateItem',
    value: function updateItem(id) {
      var _this3 = this;

      return function (path, func) {
        _this3.listener(_this3.updateIn(['items', id].concat(_toConsumableArray(path)), func));
      };
    }
  }, {
    key: 'setSelectedItem',
    value: function setSelectedItem(id) {
      if (this.selectedItem === id) return;
      this.listener(this.set('selectedItem', id));
    }
  }, {
    key: 'getSelectedItem',
    value: function getSelectedItem() {
      var item = this.items.get(this.selectedItem);
      return item ? item.toJS() : null;
    }
  }, {
    key: 'toRaw',
    value: function toRaw() {
      return this.items.toJS();
    }
  }]);

  return LayoutState;
}((0, _immutable.Record)({ items: defaultItems, selectedItem: null }));

LayoutState.fromRaw = function (raw) {
  var layoutState = new LayoutState();
  return layoutState.set('items', (0, _immutable.fromJS)(raw));
};

// LayoutState.fromJS = js => {
//   const state = {};
//   Object.keys(js).forEach(key => {
//     state[key] = new Item(fromJS(js[key]));
//   });
//   return Map(state);
// };

exports.default = LayoutState;